from datetime import datetime
from flask import Flask
from flask import request
from flask import redirect
from flask import render_template
from flask import send_from_directory
from flask import json
from logging.config import dictConfig
import os

# Erlaubte Werte für zone_door_status 
zone_door_status_values = ['OPEN', 'CLOSED', 'UNKNOWN']

# Aktueller Status
zone_door_status = 'UNKNOWN'

# Letztes Update des Status 
zone_door_lastupdate = datetime.min

# Maximales ALter (in Minuten) für den Status 
zone_door_maxage = 15

# Update Key für Updates 
setstatus_update_key = os.getenv("UPDATE_KEY", "development_key")

dictConfig({
    'version': 1,
    'formatters': {'default': {
        'format': '[%(asctime)s] %(levelname)s in %(module)s: %(message)s',
    }},
    'handlers': {'wsgi': {
        'class': 'logging.StreamHandler',
        'stream': 'ext://flask.logging.wsgi_errors_stream',
        'formatter': 'default'
    }},
    'root': {
        'level': 'INFO',
        'handlers': ['wsgi']
    }
})

app = Flask(__name__, static_url_path='')

if __name__ == "__main__":
    app.run()

# Effektiver Status 
# Wenn der letzte Status zu alt ist wird UNKNOWN zurückgegeben 
def effective_zone_door_status(app):
    global zone_door_status   
    global zone_door_lastupdate

    zone_door_alter = (datetime.now() - zone_door_lastupdate).total_seconds() / 60 
    zone_door_alter_str = str(round(zone_door_alter,2))

    app.logger.info("Current Zone Status = " + zone_door_status + " / Age = " + zone_door_alter_str + " Minutes / MaxAge = " + str(zone_door_maxage))

    if zone_door_alter > zone_door_maxage:
      return 'UNKNOWN'
    else:
      return zone_door_status  

# CORS Header setzen     
@app.after_request 
def after_request(response):
    header = response.headers
    header['Access-Control-Allow-Origin'] = '*'
    return response

# Startseite
@app.route('/')
def view_index():
    global zone_door_lastupdate
    zone_door_status = effective_zone_door_status(app)
    zone_door_alter = (datetime.now() - zone_door_lastupdate).total_seconds() / 60 
    zone_door_alter_str = str(round(zone_door_alter,2))
    return 'Warpzone API, Status = ' + zone_door_status + ', letztes Update vor ' + zone_door_alter_str + ' Minuten, see: https://gitlab.warpzone.ms/infrastruktur/warpapi/blob/master/README.md'

# Statische Dateien 
@app.route('/files/<path:path>')
def file(path):
    return send_from_directory('files', path)

# Statische Dateien 
@app.route('/setstatus')
def setstatus():
    global zone_door_status
    global zone_door_status_values
    global zone_door_lastupdate
    global setstatus_update_key

    if not 'newstatus' in request.args: 
      return "Error: Parameter 'newstatus' is missing", 500

    if not 'update_key' in request.args: 
      return "Error: Parameter 'update_key' is missing", 500

    newstatus = request.args['newstatus']
    update_key = request.args['update_key']

    app.logger.info("Update Request, newstatus = " + newstatus + " / update_key = " + update_key)

    if newstatus not in zone_door_status_values:
      app.logger.error("Update Request, newstatus is invalid")  
      return "Error: Parameter 'newstatus' is invalid", 500

    if update_key != setstatus_update_key:
      app.logger.error("Update Request, update_key is invalid, expected = " + setstatus_update_key)  
      return "Error: 'update_key' is invalid", 500 

    app.logger.info("Update Request is valid, Update Zone Status to " + newstatus)

    zone_door_status = newstatus
    zone_door_lastupdate = datetime.now();
    return "OK, Status is set to: " + zone_door_status

# Statusanzeige auf der Webseite 
@app.route('/static')
def view_static():
    spaceopen = False
    if effective_zone_door_status(app) == "OPEN":
      spaceopen = True
    return render_template('static.html', open = spaceopen)

# Statusanzeige auf der Webseite (Wordpress Widget)
@app.route('/statuswidget')
def data_status2():
    data = {
        "zone_door_status": effective_zone_door_status(app),
    } 
    response = app.response_class(
        response=json.dumps(data),
        status=200,
        mimetype='application/json'
    )
    return response

# Statusanzeige auf der Webseite (alte Version für Kompatibilität)
@app.route('/status')
def data_status():
    spaceopen = 0
    if effective_zone_door_status(app) == "OPEN":
      spaceopen = 1
    data = {
        "tuerOffen": spaceopen,
        "tempLounge": 250,
        "tempWerkstatt": 250,
        "timestamp": 0,
        "age": 500
    } 
    response = app.response_class(
        response=json.dumps(data),
        status=200,
        mimetype='application/json'
    )
    return response

# Endpunkt für Einbindung in cpu.ccc.de 
# Eigentlich die SpaceAPI, aber so können einfacher Abweichungen implementiert werden
@app.route('/cccapi')
def data_cccapi():
    return redirect('/spaceapi', code=302)

# Kompatibilität für alte urls 
@app.route('/spaceapi.php')
def data_spaceapiphp():
    return redirect('/spaceapi', code=302)

# Implementierung der SpaceAPI
@app.route('/spaceapi')
def data_spaceapi():
    spaceopen = False
    if effective_zone_door_status(app) == "OPEN":
      spaceopen = True
    data = {
        "api": "0.13",
        "api_compatibility": ["14"],
        "space": "warpzone ", 
        "logo": "https://api.warpzone.ms/files/warpzone_logo.png",
        "url": "https://www.warpzone.ms",
        "location": {
            "address": "warpzone e.V., Am Hawerkamp 31 Haus G, 48155 Münster, Germany",
            "lat": 51.9436176,
            "lon": 7.6381682,
        },
        "contact": {
            "phone": "+49 251 39500271",
            "twitter": "@warpzone_ms",
            "mastodon": "@warpzone@social.bau-ha.us",
            "email": "verwaltung@warpzone.ms",
            "irc": "irc://hackint.org/#warpzone",
            "ml": "warpzone@warpzone.ms",
            "matrix": "#warpzone:matrix.warpzone.ms"
        },
        "issue_report_channels": [ 
            "email" 
        ],
        "feeds": {
            "blog": { 
                "type": "application/rss+xml",
                "url": "https://www.warpzone.ms/feed/",
            },
            "calendar": {
                "type": "application/calendar",
                "url": "https://www.google.com/calendar/ical/hc82sdjshuf1ru9f9eqcfq9bvs%40group.calendar.google.com/public/basic.ics",
            },
        },
        "state": {
            "open": spaceopen,
            "icon": {
                "open": "https://api.warpzone.ms/files/warpzone_open.png",
                "closed": "https://api.warpzone.ms/files/warpzone_closed.png",
            },
        },
    }
    
    response = app.response_class(
        response=json.dumps(data),
        status=200,
        mimetype='application/json'
    )
    return response
